#include "scpiparser.h"
#include "scpi_user.h"
 
struct scpi_parser_context ctx;
bool bDigital[NUMBEROFCHANNELS];
int  iPins[NUMBEROFCHANNELS] = {22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53};

void scpi_user_init(void) {
  struct scpi_command* digital;

  /*
   * Set up the user command tree
   *
   *  *IDN?         -> identify
   *  *RST          -> reset
   * 
   *  :DIGItal
   *    :SWitch1    -> channel 1
   *    :SWitch2    -> channel 2
   *    :SWitch3    -> channel 3
   *    :SWitch4    -> channel 4
   *    :SWitch5    -> channel 5
   *    :SWitch6    -> channel 6
   *    :SWitch7    -> channel 7
   *    :SWitch8    -> channel 8
   *    :SWitch9    -> channel 9
   *    :SWitch10   -> channel 10
   *    :SWitch11   -> channel 11
   *    :SWitch12   -> channel 12
   *    :SWitch13   -> channel 13
   *    :SWitch14   -> channel 14
   *    :SWitch15   -> channel 15
   *    :SWitch16   -> channel 16
   *    :SWitch17   -> channel 17
   *    :SWitch18   -> channel 18
   *    :SWitch19   -> channel 19
   *    :SWitch20   -> channel 20
   *    :SWitch21   -> channel 21
   *    :SWitch22   -> channel 22
   *    :SWitch23   -> channel 23
   *    :SWitch24   -> channel 24
   *    :SWitch25   -> channel 25
   *    :SWitch26   -> channel 26
   *    :SWitch27   -> channel 27
   *    :SWitch28   -> channel 28
   *    :SWitch29   -> channel 29
   *    :SWitch30   -> channel 30
   *    :SWitch31   -> channel 31
   *    :SWitch32   -> channel 32
   * 
   *    :CELL1    -> channel 1/2
   *    :CELL2    -> channel 3/4
   *    :CELL3    -> channel 5/6
   *    :CELL4    -> channel 7/8
   *    :CELL5    -> channel 9/10
   *    :CELL6    -> channel 11/12
   *    :CELL7    -> channel 13/14
   *    :CELL8    -> channel 15/16
   *    :CELL9    -> channel 17/18
   *    :CELL10   -> channel 19/20
   *    :CELL11   -> channel 21/22
   *    :CELL12   -> channel 23/24
   *    :CELL13   -> channel 25/26
   * 
   *    :POSMESS    -> channel 27/28
   *    :NEGMESS    -> channel 29/30 
   */
  scpi_register_command(ctx.command_tree, SCPI_CL_SAMELEVEL, "*IDN?", "*IDN?", identify);
  scpi_register_command(ctx.command_tree, SCPI_CL_SAMELEVEL, "*RST", "*RST", reset);
  scpi_register_command(ctx.command_tree, SCPI_CL_SAMELEVEL, "*CLS", "*CLS", reset);
  digital = scpi_register_command(ctx.command_tree, SCPI_CL_CHILD, "DIGITAL", "DIG", NULL);

  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH1?", "SW1?", get_digital_1);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH2?", "SW2?", get_digital_2);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH3?", "SW3?", get_digital_3);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH4?", "SW4?", get_digital_4);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH5?", "SW5?", get_digital_5);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH6?", "SW6?", get_digital_6);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH7?", "SW7?", get_digital_7);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH8?", "SW8?", get_digital_8);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH9?", "SW9?", get_digital_9);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH10?", "SW10?", get_digital_10);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH11?", "SW11?", get_digital_11);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH12?", "SW12?", get_digital_12);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH13?", "SW13?", get_digital_13);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH14?", "SW14?", get_digital_14);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH15?", "SW15?", get_digital_15);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH16?", "SW16?", get_digital_16);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH17?", "SW17?", get_digital_17);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH18?", "SW18?", get_digital_18);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH19?", "SW19?", get_digital_19);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH20?", "SW20?", get_digital_20);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH21?", "SW21?", get_digital_21);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH22?", "SW22?", get_digital_22);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH23?", "SW23?", get_digital_23);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH24?", "SW24?", get_digital_24);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH25?", "SW25?", get_digital_25);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH26?", "SW26?", get_digital_26);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH27?", "SW27?", get_digital_27);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH28?", "SW28?", get_digital_28);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH29?", "SW29?", get_digital_29);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH30?", "SW30?", get_digital_30);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH31?", "SW31?", get_digital_31);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH32?", "SW32?", get_digital_32);

  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH1", "SW1", set_digital_1);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH2", "SW2", set_digital_2);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH3", "SW3", set_digital_3);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH4", "SW4", set_digital_4);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH5", "SW5", set_digital_5);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH6", "SW6", set_digital_6);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH7", "SW7", set_digital_7);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH8", "SW8", set_digital_8);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH9", "SW9", set_digital_9);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH10", "SW10", set_digital_10);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH11", "SW11", set_digital_11);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH12", "SW12", set_digital_12);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH13", "SW13", set_digital_13);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH14", "SW14", set_digital_14);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH15", "SW15", set_digital_15);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH16", "SW16", set_digital_16);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH17", "SW17", set_digital_17);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH18", "SW18", set_digital_18);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH19", "SW19", set_digital_19);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH20", "SW20", set_digital_20);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH21", "SW21", set_digital_21);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH22", "SW22", set_digital_22);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH23", "SW23", set_digital_23);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH24", "SW24", set_digital_24);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH25", "SW25", set_digital_25);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH26", "SW26", set_digital_26);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH27", "SW27", set_digital_27);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH28", "SW28", set_digital_28);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH29", "SW29", set_digital_29);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH30", "SW30", set_digital_30);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH31", "SW31", set_digital_31);
  scpi_register_command(digital, SCPI_CL_CHILD, "SWITCH32", "SW32", set_digital_32);
}

/*
 * Respond to *IDN?
 */
scpi_error_t identify(struct scpi_parser_context* context, struct scpi_token* command) {

  (void)context;  // dummy for compiler warning suppression
  (void)command;  // dummy for compiler warning suppression
  Serial.println(F("Relay control unit 1, SCPI Switch, v0.01,"));
  return SCPI_SUCCESS;
}


/*
 * Respond to *RST and *CLS
 */
scpi_error_t reset(struct scpi_parser_context* context, struct scpi_token* command) {
  int i = 0;

  (void)context;  // dummy for compiler warning suppression
  (void)command;  // dummy for compiler warning suppression

  for (i = 0; i < NUMBEROFCHANNELS; i++) {
    digitalWrite(iPins[i], LOW);
    bDigital[i] = 0;    
  }
  return SCPI_SUCCESS;
}


scpi_error_t get_digital(struct scpi_parser_context* context, struct scpi_token* command, int channel) {

  (void)context;  // dummy for compiler warning suppression
  (void)command;  // dummy for compiler warning suppression

  if (bDigital[channel] ) {
    Serial.println(F("ON"));
  } else {
    Serial.println(F("OFF"));   
  }
  return SCPI_SUCCESS;  
}


scpi_error_t set_digital(struct scpi_parser_context* context, struct scpi_token* command, int channel) {
  struct scpi_token* args;
  bool bSuccess = false;
  bool bSwitch = false;

  (void)context;  // dummy for warning suppression
//  Serial.print(F("Set digital ")); Serial.print(channel+1); Serial.print(' ');  // DEBUG

  // search last token
  args = command;
  while(args->next != NULL && args->type == 0) {
    args = args->next;
  }

//  Serial.print(args->value); Serial.print(' ');   // DEBUG
  
  if (       strcmp(args->value, "0") == 0
      || strcasecmp(args->value, "off") == 0 ) {
        
      bSuccess = true;
  } else if (       strcmp(args->value, "1") == 0 
             || strcasecmp(args->value, "on") == 0 ){
      
      bSuccess = true;
      bSwitch = true;
  }

  if (bSuccess) {
    if (bSwitch) {      
//      Serial.println(F("HIGH"));    // DEBUG
      digitalWrite(iPins[channel], HIGH);
    } else {      
//      Serial.println(F("LOW"));     // DEBUG
      digitalWrite(iPins[channel], LOW);
    }
    bDigital[channel] = bSwitch;
    return SCPI_SUCCESS;
  } else {
//    Serial.println(F("FAIL"));     // DEBUG
    return SCPI_COMMAND_NOT_FOUND;  
  }   
}

/**
 * get digital 1 status
 */
scpi_error_t get_digital_1(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 0);
}
/**
 * get digital 2 status
 */
scpi_error_t get_digital_2(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 1);
}
/**
 * get digital 3 status
 */
scpi_error_t get_digital_3(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 2);
}
/**
 * get digital 4 status
 */
scpi_error_t get_digital_4(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 3);
}
/**
 * get digital 5 status
 */
scpi_error_t get_digital_5(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 4);
}
/**
 * get digital 6 status
 */
scpi_error_t get_digital_6(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 5);
}
/**
 * get digital 7 status
 */
scpi_error_t get_digital_7(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 6);
}
/**
 * get digital 8 status
 */
scpi_error_t get_digital_8(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 7);
}
/**
 * get digital 9 status
 */
scpi_error_t get_digital_9(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 8);
}
/**
 * get digital 10 status
 */
scpi_error_t get_digital_10(struct scpi_parser_context* context, struct scpi_token* command) {
  Serial.println(F("get10"));
  return get_digital(context, command, 9);
}
/**
 * get digital 11 status
 */
scpi_error_t get_digital_11(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 10);
}
/**
 * get digital 12 status
 */
scpi_error_t get_digital_12(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 11);
}
/**
 * get digital 13 status
 */
scpi_error_t get_digital_13(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 12);
}
/**
 * get digital 14 status
 */
scpi_error_t get_digital_14(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 13);
}
/**
 * get digital 15 status
 */
scpi_error_t get_digital_15(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 14);
}
/**
 * get digital 16 status
 */
scpi_error_t get_digital_16(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 15);
}
/**
 * get digital 17 status
 */
scpi_error_t get_digital_17(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 16);
}
/**
 * get digital 18 status
 */
scpi_error_t get_digital_18(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 17);
}
/**
 * get digital 19 status
 */
scpi_error_t get_digital_19(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 18);
}
/**
 * get digital 20 status
 */
scpi_error_t get_digital_20(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 19);
}
/**
 * get digital 21 status
 */
scpi_error_t get_digital_21(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 20);
}
/**
 * get digital 22 status
 */
scpi_error_t get_digital_22(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 21);
}
/**
 * get digital 23 status
 */
scpi_error_t get_digital_23(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 22);
}
/**
 * get digital 24 status
 */
scpi_error_t get_digital_24(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 23);
}
/**
 * get digital 25 status
 */
scpi_error_t get_digital_25(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 24);
}
/**
 * get digital 26 status
 */
scpi_error_t get_digital_26(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 25);
}
/**
 * get digital 27 status
 */
scpi_error_t get_digital_27(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 26);
}
/**
 * get digital 28 status
 */
scpi_error_t get_digital_28(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 27);
}
/**
 * get digital 29 status
 */
scpi_error_t get_digital_29(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 28);
}
/**
 * get digital 30 status
 */
scpi_error_t get_digital_30(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 29);
}
/**
 * get digital 31 status
 */
scpi_error_t get_digital_31(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 30);
}
/**
 * get digital 32 status
 */
scpi_error_t get_digital_32(struct scpi_parser_context* context, struct scpi_token* command) {
  return get_digital(context, command, 31);
}

scpi_error_t set_digital_1(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 0);
}

scpi_error_t set_digital_2(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 1);
}

scpi_error_t set_digital_3(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 2);
}

scpi_error_t set_digital_4(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 3);
}

scpi_error_t set_digital_5(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 4);
}

scpi_error_t set_digital_6(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 5);
}

scpi_error_t set_digital_7(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 6);
}

scpi_error_t set_digital_8(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 7);
}

scpi_error_t set_digital_9(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 8);
}

scpi_error_t set_digital_10(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 9);
}

scpi_error_t set_digital_11(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 10);
}

scpi_error_t set_digital_12(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 11);
}

scpi_error_t set_digital_13(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 12);
}

scpi_error_t set_digital_14(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 13);
}

scpi_error_t set_digital_15(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 14);
}

scpi_error_t set_digital_16(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 15);
}

scpi_error_t set_digital_17(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 16);
}

scpi_error_t set_digital_18(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 17);
}

scpi_error_t set_digital_19(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 18);
}

scpi_error_t set_digital_20(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 19);
}

scpi_error_t set_digital_21(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 20);
}

scpi_error_t set_digital_22(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 21);
}

scpi_error_t set_digital_23(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 22);
}

scpi_error_t set_digital_24(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 23);
}

scpi_error_t set_digital_25(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 24);
}

scpi_error_t set_digital_26(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 25);
}

scpi_error_t set_digital_27(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 26);
}

scpi_error_t set_digital_28(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 27);
}

scpi_error_t set_digital_29(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 28);
}

scpi_error_t set_digital_30(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 29);
}

scpi_error_t set_digital_31(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 30);
}

scpi_error_t set_digital_32(struct scpi_parser_context* context, struct scpi_token* command) {
  return set_digital(context, command, 31);
}
